<?xml version="1.0"?>
<xsl:stylesheet 
  version='1.0' 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  xmlns:math="http://exslt.org/math"
  extension-element-prefixes="exslt math">

  <!-- Major issues. -->
  <xsl:template name="majorissues">
  
    <xsl:choose>
    
      <!-- Time Machine disabled. -->
      <xsl:when test="/etrecheck/timemachine/autobackup != 'true'">
        <xsl:call-template name="issue">
          <xsl:with-param name="type">major</xsl:with-param>
          <xsl:with-param name="name">tmdisabled</xsl:with-param>
        </xsl:call-template>
      </xsl:when>

      <!-- Time Machine not configured. -->
      <xsl:when test="/etrecheck/timemachine/notconfigured = 'true'">
        <xsl:call-template name="issue">
          <xsl:with-param name="type">major</xsl:with-param>
          <xsl:with-param name="name">tmnotconfigured</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise/>
      
    </xsl:choose>
    
    <!-- Hardware failure. -->
    <xsl:if test="count(/etrecheck/diagnostics/event[type = 10 and code &lt; -70]) &gt; 0">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">hardwarefailure</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Battery failure. -->
    <xsl:if test="/etrecheck/hardware/batteryhealth = 'Check Battery'">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">batteryfailure</xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    
    <!-- Failing hard drive. -->
    <xsl:if test="/etrecheck/disk//errors/error or /etrecheck/virtualvolume//errors/error">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">diskerrors</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <xsl:call-template name="checkdiskfail"/>
    
    <!-- Corrupt filesystem shutdown. -->
    <xsl:if test="count(/etrecheck/diagnostics/event[type = 10 and code = -60]) &gt; 0">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">corruptfilesystemshutdown</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Proxies. -->
    <xsl:if test="/etrecheck//networkinterface//proxy or /etrecheck//networkinterface/proxyautodiscovery = 'true' or /etrecheck//networkinterface/proxyautoconfig = 'true'">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">proxies</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Adware. -->
    <xsl:if test="/etrecheck/adware//launchdfile or /etrecheck/adware//extension">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">adware</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Modified suoders file. -->
    <xsl:if test="/etrecheck/configurationfiles/unexpectedsudoerssize">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">modifiedsudoers</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Gatekeeper disabled. -->
    <xsl:choose>
      <xsl:when test="/etrecheck/security/gatekeeper/status = 'Mac App Store and identified developers'">
      </xsl:when>
      
      <xsl:when test="/etrecheck/security/gatekeeper/status = 'Mac App Store'">
      </xsl:when>

      <xsl:otherwise>
        <xsl:call-template name="issue">
          <xsl:with-param name="type">major</xsl:with-param>
          <xsl:with-param name="name">gatekeeperdisabled</xsl:with-param>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>

    <!-- SIP disabled. -->
    <xsl:choose>
      <xsl:when test="/etrecheck/security/SIP = 'enabled'">
      </xsl:when>
      <xsl:when test="/etrecheck/systemsoftware/majorversion &lt; 11">
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:call-template name="issue">
          <xsl:with-param name="type">major</xsl:with-param>
          <xsl:with-param name="name">sipdisabled</xsl:with-param>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>

    <!-- Outdated security updates. -->
    <xsl:if test="/etrecheck/security/outdated = 'true'">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">securityoutdated</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Old operating system. -->
    <xsl:if test="/etrecheck/systemsoftware/majorversion &lt; 11">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">oldos</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Kernel panics. -->
    <xsl:if test="count(/etrecheck/diagnostics/event[type = 6]) &gt; 1">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">kernelpanics</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Stuck iCloud. -->
    <xsl:if test="count(/etrecheck/network/icloudpendingfiles//file) &gt; 10">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">icloudstuck</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Poor performance. -->
    <xsl:if test="/etrecheck/header/performance = 'poorperformance'">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">poorperformance</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Heavy CPU usage. -->
    <xsl:if test="sum(/etrecheck/cpu/process/usage) &gt; 150">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">heavycpu</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Low disk space. < 10 GB -->
    <xsl:variable name="freespace">
      <xsl:call-template name="expand">
        <xsl:with-param 
          name="number" 
          select="/etrecheck//volume[mountpoint = '/']/free"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:if test="$freespace &lt; 10000000000">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">lowdiskspace</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Low RAM. -->
    <xsl:if test="sum(/etrecheck/hardware/ram) &lt; 4">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">lowram</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Obsolete system. -->
    <xsl:if test="/etrecheck/hardware/vintage = 'obsolete'">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">major</xsl:with-param>
        <xsl:with-param name="name">obsolete</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

  </xsl:template>

  <!-- Check for disk failure. -->
  <xsl:template name="checkdiskfail">
  
    <xsl:if test="/etrecheck/disk/drive[device = 'disk0']">
      <xsl:if test="/etrecheck/disk/drive[solidstate = 'false']">
        <xsl:if test="/etrecheck/header/performance = 'Poor'">
          <xsl:if test="$diskfail = 'true'">
            <xsl:call-template name="issue">
              <xsl:with-param name="type">major</xsl:with-param>
              <xsl:with-param name="name">diskfail</xsl:with-param>
            </xsl:call-template>
          </xsl:if>
        </xsl:if>
      </xsl:if>
    </xsl:if>
      
  </xsl:template>
    
  <!-- Minor issues. -->
  <xsl:template name="minorissues">
    
    <!-- Low disk space. < 20 GB -->
    <xsl:call-template name="freespace"/>

    <!-- Check various options for upgradeability. -->
    <xsl:call-template name="checkupgradeability"/>
    
    <!-- Heavy RAM usage. -->
    <xsl:call-template name="heavyram"/>

    <!-- Heavy network usage and recent boot usage. -->
    <xsl:call-template name="heavynetwork"/>
    
    <!-- Killed launchd tasks with mechanical hard drive. -->
    <xsl:call-template name="killedlaunchd"/>

    <!-- Old machine. -->
    <xsl:call-template name="oldmachine"/>
    
    <!-- High battery cycle count. -->
    <xsl:if test="/etrecheck/hardware/batterypercent &gt; 70">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">highbatterycycles</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Clean up. -->
    <xsl:if test="count(/etrecheck/cleanup//launchdfile) &gt; 0 or count(/etrecheck/cleanup//extension) &gt; 0">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">cleanup</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Small Time machine drive. -->
    <xsl:call-template name="checktmsize"/>

    <!-- Unsigned files. -->
    <xsl:if test="/etrecheck/unsigned//launchdfile[apple != 'true']">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">unsigned</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Large number of system modifications. -->
    <xsl:if test="count(/etrecheck/launchagents/launchdfile) &gt; 10 or count(/etrecheck/launchdeamons/launchdfile) &gt; 10">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">systemmodifications</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Below average performance. -->
    <xsl:if test="/etrecheck/header/performance = 'belowaverageperformance'">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">belowaverageperformance</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <!-- Corrupt hosts file. -->
     <xsl:if test="/etrecheck/configurationfiles/hostscorrupt">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">corrupthosts</xsl:with-param>
      </xsl:call-template>
    </xsl:if>
 
    <!-- Vintage system. -->
    <xsl:if test="/etrecheck/hardware/vintage = 'vintage'">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">vintage</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

    <xsl:if test="/etrecheck/diagnostics/insufficientpermissions = 'true'">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">insufficientpermissions</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

  </xsl:template>

  <!-- Check for low free disk space. -->
  <xsl:template name="freespace">
  
    <xsl:variable name="freespace">
      <xsl:call-template name="expand">
        <xsl:with-param 
          name="number" 
          select="/etrecheck//volume[mountpoint = '/']/free"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:if test="$freespace &gt;= 10000000000 and $freespace &lt; 20000000000">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">lowdiskspace</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

  </xsl:template>

  <!-- Check various options for upgradeability. -->
  <xsl:template name="checkupgradeability">
  
    <!-- Upgradeable HD. -->
    <xsl:variable name="upgradeablehd">
      <xsl:call-template name="upgradeablehd"/>
    </xsl:variable>
    
    <!-- Underpowered. -->
    <xsl:variable name="underpowered">
      <xsl:call-template name="underpowered"/>
    </xsl:variable>
    
    <xsl:choose>

      <xsl:when test="$upgradeablehd = 'true'">
        <xsl:call-template name="issue">
          <xsl:with-param name="type">minor</xsl:with-param>
          <xsl:with-param name="name">upgradeablehd</xsl:with-param>
        </xsl:call-template>
      </xsl:when>

      <xsl:when test="/etrecheck/hardware/upgradeable = 'true' and /etrecheck/hardware/ram &lt; 8">
        <xsl:call-template name="issue">
          <xsl:with-param name="type">minor</xsl:with-param>
          <xsl:with-param name="name">upgradeableram</xsl:with-param>
        </xsl:call-template>
      </xsl:when>

      <xsl:when test="$underpowered = 'true'">
        <xsl:call-template name="issue">
          <xsl:with-param name="type">minor</xsl:with-param>
          <xsl:with-param name="name">underpowered</xsl:with-param>
        </xsl:call-template>
      </xsl:when>

      <xsl:otherwise/>
    </xsl:choose>
      
  </xsl:template>

  <!-- Underpowered machine. -->
  <xsl:template name="underpowered">
  
    <xsl:variable 
      name="bootdisk" select="/etrecheck/disk/drive[device = 'disk0']"/>
    
    <xsl:choose>
      <xsl:when test="$bootdisk[solidstate = 'false'] and /etrecheck/hardware/ram &lt; 8 and /etrecheck/hardware/upgradeable = 'false'">
        <xsl:value-of select="true()"/>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Upgradeable hard drive. -->
  <xsl:template name="upgradeablehd">
  
    <xsl:choose>
      <xsl:when test="/etrecheck/hardware/upgradeable = 'true' and /etrecheck/hardware/ram &lt; 8">
        <xsl:choose>
          <xsl:when test="/etrecheck/hardware/modeltype = 'MacBookPro'">
            <xsl:value-of select="true()"/>
          </xsl:when>
          
          <xsl:when test="/etrecheck/hardware/modeltype = 'MacMini'">
            <xsl:value-of select="true()"/>
          </xsl:when>

          <xsl:otherwise>
            <xsl:value-of select="false()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for heavy ram usage. -->
  <xsl:template name="heavyram">
  
    <xsl:variable 
      name="kernel_task" 
      select="/etrecheck/memory/process[name = 'kernel_task']"/>
    
    <xsl:variable
      name="other"
      select="/etrecheck/memory/process[name != 'kernel_task' and size/@units = 'GB' and size &gt; 2]"/>
      
    <xsl:variable
      name="heavyram">
      <xsl:choose>
        <xsl:when test="$kernel_task/size[@units = 'GB'] &gt; (/etrecheck/hardware/ram * 0.2)">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when test="count($other) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
            
        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:if test="$heavyram = 'true'">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">heavyram</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

  </xsl:template>

  <!-- Check for heavy network usage. -->
  <xsl:template name="heavynetwork">
  
    <xsl:variable
      name="heavynetwork"
      select="/etrecheck/networkusage/process[inputsize/@units = 'GB' or outputsize/@units = 'GB']"/>
      
    <xsl:choose>
      <xsl:when test="count($heavynetwork) &gt;= 1 and /etrecheck/systemsoftware/uptime &lt; 24">
        <xsl:call-template name="issue">
          <xsl:with-param name="type">minor</xsl:with-param>
          <xsl:with-param name="name">heavynetwork</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
          
      <xsl:otherwise/>
    </xsl:choose>
    
  </xsl:template>

  <!-- Check for killed launchd tasks and a mechanical hard drive. -->
  <xsl:template name="killedlaunchd">
  
    <xsl:variable
      name="killedlaunchd"
      select="/etrecheck/systemlaunchagents/launchdfile[status = 'killed'] | /etrecheck/systemlaunchdaemons/launchdfile[status = 'killed']"/>
      
    <xsl:variable 
      name="bootdisk" select="/etrecheck/disk/drive[device = 'disk0']"/>
    
    <xsl:choose>
      <xsl:when test="$bootdisk[solidstate = 'false'] and count($killedlaunchd) &gt; 0">
        <xsl:call-template name="issue">
          <xsl:with-param name="type">minor</xsl:with-param>
          <xsl:with-param name="name">killedlaunchd</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise/>
    </xsl:choose>

  </xsl:template>

  <!-- Check for an old machine with a new OS. -->
  <xsl:template name="oldmachine">
  
    <xsl:variable
      name="newos"
      select="/etrecheck/systemsoftware/majorversion &gt; 9"/>
      
    <xsl:variable
      name="oldmachine">
      <xsl:choose>
        <xsl:when test="/etrecheck/hardware/modeltype = 'MacBook'">
          <xsl:choose>
            <xsl:when test="/etrecheck/hardware/majorversion &lt; 5">
              <xsl:value-of select="true()"/>
            </xsl:when>
            
            <xsl:otherwise>
              <xsl:value-of select="false()"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        
        <xsl:when test="/etrecheck/hardware/modeltype = 'MacBookAir'">
          <xsl:choose>
            <xsl:when test="/etrecheck/hardware/majorversion &lt; 3">
              <xsl:value-of select="true()"/>
            </xsl:when>
            
            <xsl:otherwise>
              <xsl:value-of select="false()"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>

        <xsl:when test="/etrecheck/hardware/modeltype = 'MacBookPro'">
          <xsl:choose>
            <xsl:when test="/etrecheck/hardware/majorversion &lt; 5">
              <xsl:value-of select="true()"/>
            </xsl:when>
            
            <xsl:otherwise>
              <xsl:value-of select="false()"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>

        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
        
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$oldmachine = 'true' and $newos = 'true'">
    
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">oldmachine</xsl:with-param>
      </xsl:call-template>
            
    </xsl:if>
    
  </xsl:template>

  <!-- Check for Time Machine drive too small. -->
  <xsl:template name="checktmsize">
  
    <xsl:variable name="volumesizes">
      <xsl:for-each select="/etrecheck/timemachine//volume">
        <xsl:variable name="value">
          <xsl:call-template name="expand">
            <xsl:with-param name="number" select="size"/>
          </xsl:call-template>
        </xsl:variable>
        <value>
        <xsl:value-of select="$value div 1024 div 1024 div 1024"/>
        </value>
      </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="destinationsizes">
      <xsl:for-each select="/etrecheck/timemachine//destination">
        <xsl:variable name="value">
          <xsl:call-template name="expand">
            <xsl:with-param name="number" select="size"/>
          </xsl:call-template>
        </xsl:variable>
        
        <value>
        <xsl:value-of select="$value div 1024 div 1024 div 1024"/>
        </value>
      </xsl:for-each>
    </xsl:variable>

    <xsl:variable 
      name="maxvolumesize" select="math:max(exslt:node-set($volumesizes)/value)"/>
    
    <xsl:variable 
      name="maxdestinationsize" 
      select="math:max(exslt:node-set($destinationsizes)/value)"/>
    
    <xsl:if test="$maxdestinationsize &lt; ($maxvolumesize * 2)">
      <xsl:call-template name="issue">
        <xsl:with-param name="type">minor</xsl:with-param>
        <xsl:with-param name="name">tmtoosmall</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

  </xsl:template>
  
</xsl:stylesheet>
